function sol = lms(param, ode, h)
%LMS Solve ordinary differential equation with linear multistep method.
% PARAM = LMSPARAMS(ALPHA, BETA); SOL = LMS(PARAM, ODE, H) 
% integrates ODE with constant step size h, linear 
% multistep method sum(alpha(k)y(n-k) = h*sum(beta(k)f(n-k)).
% Alpha and beta are vectors of the same length, f is function in t and y.
% FCALLS is the number of calls of F and DF, minus some in the
% initialization process.

% created by Claude Gittelson

sol.title = sprintf('%s solved with %s, step size %g', ode.name, param.name, h);

sol.method = param;
sol.ode = ode;
sol.h = h;

tic; % start stopwatch

if(param.alpha(1)==0) % alpha(1) not zero
    return;
end

sol.t = [ode.t0:h:ode.tN]; % time, independant variable
if(length(sol.t)<=max(param.k,1))
    return;
end
sol.y = zeros(ode.dim,length(sol.t)); % solution, dependant variable
F = sol.y;  % evaluations of f
yn = zeros(ode.dim,2); % iteration for y(n)
if(nargin>=4) % tolerance for iterations in implicit methods
    sol.tol = tol;
else
    sol.tol = 1e-10;
end
sol.fcalls = 0; % number of function calls

% initial condition
sol.y(:,1) = ode.y0;
F(:,1) = feval(ode.f,sol.t(1),sol.y(:,1),ode.fargs{:});
sol.fcalls = sol.fcalls + ode.dim;

% initialize with ode45
if(param.k>=2)
	solInit = ode45(ode.f, [sol.t(1), sol.t(param.k)], ode.y0, [], ode.fargs{:});
	for n = 2:param.k
        sol.y(:,n) = deval(solInit, sol.t(n));
        F(:,n) = feval(ode.f,sol.t(n),sol.y(:,n),ode.fargs{:});
        sol.fcalls = sol.fcalls + pde.dim;
	end
end

% continue with multistep method
for n = param.k+1:length(sol.t)
    if(param.explicit)
        sol.y(:,n) = (-sol.y(:,n-1:-1:n-param.k)*param.alpha(2:param.k+1) + h*F(:,n-1:-1:n-param.k)*param.beta(2:param.k+1))/param.alpha(1);
        F(:,n) = feval(ode.f,sol.t(n),sol.y(:,n),ode.fargs{:});
        sol.fcalls = sol.fcalls + ode.dim;
    else
		% newton iteration
		yn(:,1) = sol.y(:,n-1);
		const = sol.y(:,n-1:-1:n-param.k)*param.alpha(2:param.k+1) - h*F(:,n-1:-1:n-param.k)*param.beta(2:param.k+1);
		d = zeros(ode.dim,1);
		m = 1;
		a = 2; b = 1; c = 1;
        [L,U,P] = lu(param.alpha(1)*eye(ode.dim) - h*param.beta(1)*feval(ode.Df,sol.t(n),yn(:,b),ode.fargs{:}));
% 		warning off MATLAB:nearlySingularMatrix;
		while (norm(d)>sol.tol | m==1) & m<100 
			g = param.alpha(1)*yn(:,b) - h*param.beta(1)*feval(ode.f,sol.t(n),yn(:,b),ode.fargs{:}) + const;
            if rem(m,10) == 0
                [L,U,P] = lu(param.alpha(1)*eye(ode.dim) - h*param.beta(1)*feval(ode.Df,sol.t(n),yn(:,b),ode.fargs{:}));
                sol.fcalls = sol.fcalls + ode.dim^2;
            end
            d = U\(L\(P*(-g)));
			yn(:,a) = yn(:,b) + d;
			c = a; a = b; b = c; m = m+1;
		end
		sol.y(:,n) = yn(:,c);
		F(:,n) = feval(ode.f,sol.t(n),sol.y(:,n),ode.fargs{:});
        sol.fcalls = sol.fcalls + m*ode.dim + ode.dim^2;
    end
end

sol.time = toc; % read stopwatch